/*
  LcdTime.c

  release : 0.0.1 - 23 Jan 2013

  ------------------------------------------------------------------------------
  Copyright (c) 2011, 2013 Wim Langers.  All right reserved.

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
  ------------------------------------------------------------------------------
 */

#include <Arduino.h>
#include "LcdTime.h"

static uint32_t millisOld;

/*
  Initialise timer.
*/
void LcdTimeInit() {
  millisOld = millis();
}

/*
  Refresh date/time.
  Must be called cyclically to refresh date and time.

  parameters :
  dt        current date/time
  return    0 if time not changed
            number of seconds lapsed otherwise
*/
uint16_t LcdTimeRefresh(LcdTime* dt) {
  uint16_t res = 0;
  while (((millis() - millisOld) & 0x7FFFFFFF) > 1000) {
    ++res;
    millisOld += 1000;
    if (dt->second < 59) ++dt->second;
    else {
      dt->second = 0;
      if (dt->minute < 59) ++dt->minute;
      else {
        dt->minute = 0;
        if (dt->hour < 23) ++dt->hour;
        else {
          dt->hour = 0;
          if (dt->day < LcdTimeDaymax(dt)) ++dt->day;
          else {
            dt->day = 1;
            if (dt->month < 12) ++dt->month;
            else {
              dt->month = 1;
              ++dt->year;
            }
          }
        }
      }
    }
  }
  return res;
}

/*
  Maximum number of days in month.

  parameters :
  dt        date and time
  return    maximum day in month
*/
uint8_t LcdTimeDaymax(LcdTime* dt) {
  switch(dt->month) {
    case 1: case 3: case 5: case 7: case 8: case 10: case 12: return 31;
    case 4: case 6: case 9: case 11: return 30;
    case 2: return (dt->year % 4 == 0) ? 29 : 28;
  }
}

/*
  Week day.

  parameters :
  dt        date and time
  return    day of the week (1 = monday..7 = sunday)
  remark :
  1/1/2000 = saturday, first legal date = 1/1/2001 !!!
*/
uint8_t LcdTimeWeekday(LcdTime* dt) {
  uint16_t days = dt->year * 365 + dt->year / 4 + 1;
  switch(dt->month - 1) {
    case 11: days += 30;
    case 10: days += 31;
    case 9: days += 30;
    case 8: days += 31;
    case 7: days += 31;
    case 6: days += 30;
    case 5: days += 31;
    case 4: days += 30;
    case 3: days += 31;
    case 2: days += (dt->year % 4 == 0) ? 29 : 28;
    case 1: days += 31;
  }
  days += dt->day;
  return (uint8_t)(days + 4) % 7 + 1;
}
